/*****************************************************************
 * File Name:	wmudflib.c
 * Description:
 * This module contains some user defined functions to operate on blobs.
 * Originally based on the udflib.c found in the interbase examples v5 
 * directory but completely rewritten because it didn't seem thread safe.
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code was created by Inprise Corporation
 * and its predecessors. Portions created by Inprise Corporation are
 * Copyright (C) Inprise Corporation.
 *
*****************************************************************/

#include <string.h>
#include <ibase.h>
#include "example.h"

//****************************************************************

typedef struct blob {
	short	(*blob_get_segment) ();
	void	*blob_handle;
	long	blob_number_segments;
	long	blob_max_segment;
	long	blob_total_length;
	void	(*blob_put_segment) ();
} *BLOB;

//****************************************************************
long EXPORT fn_blob_length (ARG(BLOB, sourceBlob))
ARGLIST(BLOB sourceBlob) { 
//****************************************************************
// fn_blob_length() returns the number of bytes in a blob 
//****************************************************************
  if (!sourceBlob->blob_handle) {
    return 0L;
  }
  return (sourceBlob->blob_total_length);  
}

//****************************************************************
char* EXPORT fn_blob_substr(ARG(BLOB, sourceBlob), ARG(long*, startPos), ARG(long*, endPos), ARG(char*, sResult))
ARGLIST(BLOB sourceBlob)
ARGLIST(long *startPos)
ARGLIST(long *endPos)
ARGLIST(char *sResult) {
//****************************************************************
// fn_blob_substr() returns portion of TEXT blob beginning at the 
// startPos character and ended at the endPos character. 
//****************************************************************
  char *pbuffer, *pOffset, *pResultOffset;
  long i = 0;
  long curr_bytecount = 0;
  long startChar, endChar; 
  long length, actual_length;
  *sResult = 0;
  if (!sourceBlob->blob_handle) {
    return sResult;
  }
  length = sourceBlob->blob_max_segment;
  if (*startPos > *endPos || *startPos < 1L || *endPos < 1L)  {
    return sResult;
  }
  if (sourceBlob->blob_total_length < (long)*startPos) {
    return sResult;
  }
  startChar = *startPos;
  if (sourceBlob->blob_total_length < (long)*endPos) {
    endChar = sourceBlob->blob_total_length;
  } else {
    endChar = *endPos;
  }
  pbuffer = (char *) malloc (length + 1L); 
  pResultOffset = sResult;
  while ((*sourceBlob->blob_get_segment) (sourceBlob->blob_handle, pbuffer, length, &actual_length)) {    
//    pbuffer [actual_length] = 0;
    pOffset = pbuffer;
    while (*pOffset && (curr_bytecount < endChar)) {
      curr_bytecount++;
      if (curr_bytecount >= startChar) {
	*pResultOffset++ = *pOffset;
      }
      pOffset++;
    }
    if (curr_bytecount >= endChar) {
      *pResultOffset = 0;
      break;
    }
  }
  free (pbuffer);
  return sResult;
}

//****************************************************************
char* EXPORT fn_blob_string(ARG(BLOB, sourceBlob), ARG(char*, sResult))
ARGLIST(BLOB sourceBlob)
ARGLIST(char *sResult) {
//****************************************************************
// fn_blob_string() returns the contents of TEXT blob beginning at the 
// 1st character and ended at the last character. 
//****************************************************************
  long start;
  long end;
  start = 1;
  end = fn_blob_length(sourceBlob);
  fn_blob_substr(sourceBlob, &start, &end, sResult);
  return sResult;
}

//****************************************************************
char* EXPORT fn_blob_left(ARG(BLOB, sourceBlob), ARG(long*, characters), ARG(char*, sResult))
ARGLIST(BLOB sourceBlob)
ARGLIST(long *characters)
ARGLIST(char *sResult) {
//****************************************************************
// fn_blob_left() returns the contents of TEXT blob beginning at the 
// 1st character and ended at the characters character. 
//****************************************************************
  long start;
  long end;
  start = 1;
  end = *characters;
  fn_blob_substr(sourceBlob, &start, &end, sResult);
  return sResult;
}

//****************************************************************
char* EXPORT fn_blob_right(ARG(BLOB, sourceBlob), ARG(long*, characters), ARG(char*, sResult))
ARGLIST(BLOB sourceBlob)
ARGLIST(long *characters)
ARGLIST(char *sResult) {
//****************************************************************
// fn_blob_right() returns 'characters' from the right of the 
// TEXT blob beginning at the 
//****************************************************************
  long start;
  long end;
  start = fn_blob_length(sourceBlob) - *characters + 1;
  if (start < 1) {
    start = 1;
  }
  end = fn_blob_length(sourceBlob);
  fn_blob_substr(sourceBlob, &start, &end, sResult);
  return sResult;
}

//****************************************************************
BLOB EXPORT fn_string_blob(ARG(char*, sourceString), ARG(BLOB, sResult))
ARGLIST(char *sourceString) 
ARGLIST(BLOB sResult) {
//****************************************************************
// fn_string_blob() returns a blob containing the contents of the 
// source string. 
//****************************************************************
  (*sResult->blob_put_segment) (sResult->blob_handle, sourceString, strlen(sourceString));
  return sResult;
}

//****************************************************************
BLOB EXPORT fn_blob_string_append(ARG(BLOB, sourceBlob), ARG(char*, sourceString), ARG(BLOB, sResult))
ARGLIST(BLOB sourceBlob)
ARGLIST(char *sourceString) 
ARGLIST(BLOB sResult) {
//****************************************************************
// fn_string_blob() returns the source blob with the source string 
// appended to the end. 
//****************************************************************
  long stringlength;
  char tempString[(sourceBlob->blob_total_length) + strlen(sourceString) + 1L];
  long start;
  long end;
  start = 1;
  end = fn_blob_length(sourceBlob);
  fn_blob_substr(sourceBlob, &start, &end, tempString);
  strcat(tempString, sourceString);
  (*sResult->blob_put_segment) (sResult->blob_handle, tempString, strlen(tempString));
  return sResult;
}

//****************************************************************
long EXPORT fn_blob_compare (ARG(BLOB, blobA), ARG(BLOB, blobB))
ARGLIST(BLOB blobA)
ARGLIST(BLOB blobB) {
//****************************************************************
// Compares two blobs together and checks for differences.
// Returns -1 if the blobs are the same or the position where they differ.
//****************************************************************
  long counterA = 0;
  long counterB = 0;
  long counter = 0;
  char * bufferA;
  char * bufferB;
  int maxA;
  int maxB;
  long blobASize = (blobA->blob_total_length);
  long blobBSize = (blobB->blob_total_length);
  if ((blobASize == 0) && (blobBSize == 0)) {
    return 0L;
  }
  maxA = blobA->blob_max_segment + 1L;
  bufferA = (char *) malloc (maxA); 
  maxB = blobB->blob_max_segment + 1L;
  bufferB = (char *) malloc (maxB); 
  maxA = 0;
  maxB = 0;
  while ((counter < blobASize) && (counter < blobBSize)) {
    if (counterA >= maxA) {
      (*blobA->blob_get_segment) (blobA->blob_handle, bufferA, blobA->blob_max_segment, &maxA);      
      counterA = 0;
    }
    if (counterB >= maxB) {
      (*blobB->blob_get_segment) (blobB->blob_handle, bufferB, blobB->blob_max_segment, &maxB);      
      counterB = 0;
    }
    if (*(bufferA + counterA) != *(bufferB + counterB)) {
      free(bufferA);
      free(bufferB);
      return counter + 1L;
    }
    counter++;
    counterA++;
    counterB++;
  } 
  free(bufferA);
  free(bufferB);
  if (blobASize != blobBSize) {
    return counter + 1L;
  }
  return 0L;
}

//****************************************************************
BLOB EXPORT fn_blob_string_insert(ARG(BLOB, sourceBlob), ARG(char*, sourceString), ARG(long*, position), ARG(BLOB, sResult))
ARGLIST(BLOB sourceBlob)
ARGLIST(char *sourceString) 
ARGLIST(long *position)
ARGLIST(BLOB sResult) {
//****************************************************************
// fn_blob_string_insert() returns the source blob with the source string 
// inserted at position. 
//****************************************************************
  long stringLength;
  long start;
  long end;
  long i;
  char tempString[(sourceBlob->blob_total_length) + strlen(sourceString) + 1L];
  *position = *position -1L;
  start = 1;
  end = (sourceBlob->blob_total_length);
  if (*position > end) {
    *position = end;
  }
  stringLength = strlen(sourceString);
  fn_blob_substr(sourceBlob, &start, &end, tempString);
  for (i=end;i>*position -1;i--) {
    tempString[i + stringLength] = tempString[i];
  }
  for (i=0;i< stringLength;i++) {
    tempString[i + *position] = *(sourceString + i);
  }
  (*sResult->blob_put_segment) (sResult->blob_handle, tempString, strlen(tempString));
  return sResult;
}

//****************************************************************
BLOB EXPORT fn_blob_delete_chars(ARG(BLOB, sourceBlob), ARG(long*, startPos), ARG(long*, endPos), ARG(BLOB, sResult))
ARGLIST(BLOB sourceBlob)
ARGLIST(long *startPos)
ARGLIST(long *endPos)
ARGLIST(BLOB sResult) {
//****************************************************************
// fn_blob_delete_chars() returns the source blob with the the 
// characters starting at startPos to endPos removed.
//****************************************************************
  long stringlength;
  long start;
  long end;
  long i;
  char tempString[(sourceBlob->blob_total_length) + 1L];
  if (*endPos < *startPos) {
    return;
  }
  start = 1;
  end = (sourceBlob->blob_total_length);
  fn_blob_substr(sourceBlob, &start, &end, tempString);
  for (i=*startPos -1;i < end;i++) {
    tempString[i] = tempString[i + *endPos - *startPos + 1];
  }
  (*sResult->blob_put_segment) (sResult->blob_handle, tempString, strlen(tempString));
  return sResult;
}

//****************************************************************
long EXPORT fn_blob_string_pos (ARG(BLOB, sourceBlob), ARG(char*, compStr))
ARGLIST(BLOB sourceBlob)
ARGLIST(char *compStr) {
//****************************************************************
// Compares two blobs together and checks for differences.
// Returns -1 if the blobs are the same or the position where they differ.
//****************************************************************
  long counterA = 0;
  long counter = 0;
  long blobSize = sourceBlob->blob_total_length;
  int strSize = strlen(compStr);
  char tempString[(blobSize) + 1];
  int i;
 
  if (blobSize == 0) {
    return 0L;
  }
  fn_blob_string(sourceBlob, tempString);
  for (i=0;i < (blobSize - strSize + 1); i++) {
    if (strncmp(&tempString[i], compStr, strSize) == 0) {
      return i + 1L;
    }
  }
  return 0L;
}
